/*******************************************
REGISTER UNIT
All registers execute the same code,
the only difference is the I2C address
********************************************

HOMS version 1B
Hardware-Oriented Microprocessor
Simulator

(C) Panayotis (Panos) Papazoglou
https://homs.panospapazoglou.gr/

LICENSE: 
Creative Commons
CC BY NC SA
International License
********************************************/

//Change address based on block that is used
#define ADDRESS 6


//new seven segment
#include <Arduino.h>
#include <TM1637Display.h>

//Seven segment pins
#define CLK 9
#define DIO 8

TM1637Display display(CLK, DIO);
uint8_t data[] = { 0x00, 0x00, 0x00, 0x00 };

//Include I2C library
#include <Wire.h>

//Define symbolic names for instructions codes
//between blocks
#define READ 65
#define RESET 66
#define SHIFT_L 67
#define SHIFT_R 68
#define DEC 69
#define INC 70
#define LOAD 71
#define IDON 99
#define IDOFF 98
#define IDOK 97

int identify = 0;

//byte that is received via I2C communication
byte rV;

//Previous received byte. Is used when a sequence of bytes
//has to be checked
byte rVprev = 0;

//Initial register content
byte REG = 0xff;

//Starting function (after reset or power on), runs once
void setup() {

  display.setBrightness(0x0f);

  //Initialize Serial communication
  //for displaying debugging messages
  Serial.begin(9600);
  Serial.println("hello from MBR");

  //Display register content
  displayR(REG);

  //Initialize I2C communication:
  //Set address
  //Enable ISR for receiving bytes and requests
  Wire.begin(ADDRESS);
  Wire.onReceive(onReceive);
  Wire.onRequest(onRequest);
}

void loop() {
  //Nothing here
}

/*******************************************
Action Routine that is activated when 
a byte is received 
********************************************/
void onReceive(int a)

{

  //Read received byte
  rV = Wire.read();
  Serial.println("Byte received!");
  //If previous byte is 71 (LOAD), then the current byte
  //is the value that will be loaded in register
  if (rVprev == LOAD) REG = rV;

  //Update variable for previous value
  rVprev = rV;

  //Actions based on received byte
  if (rV == INC) REG++;
  if (rV == DEC) REG--;
  if (rV == SHIFT_R) REG = REG >> 1;
  if (rV == SHIFT_L) REG = REG << 1;
  if (rV == RESET) REG = 00;

  //Display register content after performed action
  displayR(REG);
}

/*******************************************
Action Routine that is activated when 
a request is received 
********************************************/
void onRequest() {

  //If the received byte represents
  //the READ command, then the
  //register content is sent as answer
  if (rV == READ) Wire.write(REG);
  if (rV == IDON) Wire.write(IDOK);


  //Display register content
  displayR(REG);
}

/*******************************************
Display register content as HEX number on
seven segment module 
Digit manipulation for left and right
seven segment display unit 
********************************************/
void displayR(byte REGnum) {

  //Convert decimal number REGnum to HEX
  String Shex = String(REGnum, HEX);

  //Variables for left and right digit
  char LL;
  char RR;

  //If the hex number has only one digit, then
  //the left digit is zero ('0') and the right digit
  //is the first character of the string Shex
  if (Shex.length() == 1) {
    LL = '0';
    RR = Shex[0];
  }

  //Otherwise, update left and right digit variables from
  //the whole string Shex
  else {
    LL = Shex[0];
    RR = Shex[1];
  }

  //variables for accessing later the hex[] array
  //in order to activate the correct segment
  //on seven segment module
  byte left, right;

  if (LL >= 'a' && LL <= 'f') left = LL - 87;
  else if (LL >= '0' && LL <= '9') left = LL - 48;

  if (RR >= 'a' && RR <= 'f') right = RR - 87;
  else if (RR >= '0' && RR <= '9') right = RR - 48;

  display2(left, right);
}

/*******************************************
Digit manipulation
final display on
seven segment unit 
********************************************/
void display2(int d1, int d3) {
  display.clear();
  data[1] = display.encodeDigit(d1);
  data[3] = display.encodeDigit(d3);
  display.setSegments(data);
}
